/* Sorting */
table.dataTable tr.odd td.sorting_1 { background-color: #D3D6FF; }
table.dataTable tr.odd td.sorting_2 { background-color: #DADCFF; }
table.dataTable tr.odd td.sorting_3 { background-color: #E0E2FF; }
table.dataTable tr.even td.sorting_1 { background-color: #EAEBFF; }
table.dataTable tr.even td.sorting_2 { background-color: #F2F3FF; }
table.dataTable tr.even td.sorting_3 { background-color: #F9F9FF; }

table.dataTable tr.odd td.sorting_1 { background-color: #D3D6FF; }
table.dataTable tr.odd td.sorting_2 { background-color: #DADCFF; }
table.dataTable tr.odd td.sorting_3 { background-color: #E0E2FF; }
table.dataTable tr.even td.sorting_1 { background-color: #EAEBFF; }
table.dataTable tr.even td.sorting_2 { background-color: #F2F3FF; }
table.dataTable tr.even td.sorting_3 { background-color: #F9F9FF; }

/* Selected rows */
table.dataTable tbody tr.selected,
table.dataTable tr.selected td.sorting_1,
table.dataTable tr.selected td.sorting_2,
table.dataTable tr.selected td.sorting_3,
.table-striped tbody>tr.selected:nth-child(odd)>td,
.table-striped tbody>tr.selected:nth-child(even)>td {
  background-color: #b0bed9;
}

/* Search boxes in the footer */
table tfoot input {
  width: 100%;
}

