@mixin no-click() {
    -webkit-user-select: none;
    -khtml-user-select: none;
    -moz-user-select: none;
    -ms-user-select: none;
    user-select: none;
}

@mixin pos-r() {
    position: relative;
    display: block;
}

@mixin pos-a() {
    position: absolute;
    display: block;
}
