// Datepicker .scss buildfile.  Includes select mixins/variables from bootstrap
// and imports the included datepicker.scss to output a minimal datepicker.css
//
// Usage:
//     lessc build3.scss datepicker.css
//
// Variables and mixins copied from Bootstrap 3.3.5

// These are BS3 variables that are used in datepicker3.scss. So, when compiling against
// a BS3 bslib theme, these variables should already be defined. Here we set
// *defaults* for these variables based on BS4 variables, so this scss can work for
// both BS3 and BS4
$gray:                mix($body-bg, $body-color, 33.5%) !default;
$gray-light:          mix($body-bg, $body-color, 46.7%) !default;
$gray-lighter:        mix($body-bg, $body-color, 90%) !default;
$brand-primary:       $primary !default;
$btn-primary-color:   $body-bg !default;
$btn-primary-bg:      $primary !default;
$btn-primary-border:  mix($body-color, $btn-primary-bg, 5%) !default;
$state-info-bg:       mix($body-bg, $info, 80%) !default;
$border-radius-base:  $border-radius !default;
$dropdown-border:     $dropdown-border-color !default;
// These variables are also used in datepicker3.scss, but we don't need to set them since
// they're the same in BS3 and BS4
//$line-height-base:        1.428571429;
//$btn-link-disabled-color: $gray-light;
//$dropdown-bg:             #fff;





@mixin button-variant($background, $border) {
  $color: color-contrast($background);

  color: $color;
  background-color: $background;
  border-color: $border;

  &:focus,
  &.focus {
    color: $color;
    background-color: mix($background, $color, 90%);
        border-color: mix($border, $color, 75%);
  }
  &:hover {
    color: $color;
    background-color: mix($background, $color, 50%);
        border-color: mix($border, $color, 88%);
  }
  &:active,
  &.active {
    color: $color;
    background-color: mix($background, $color, 90%);
        border-color: mix($border, $color, 88%);

    &:hover,
    &:focus,
    &.focus {
      color: $color;
      background-color: mix($background, $color, 83%);
          border-color: mix($border, $color, 75%);
    }
  }
  &.disabled,
  &[disabled],
  fieldset[disabled] & {
    &:hover,
    &:focus,
    &.focus {
      background-color: $background;
          border-color: $border;
    }
  }
}

@import "datepicker3.scss";
